<?php
namespace Teknisa\GER\Controllers;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;

class SearchPratoDemanda extends \Teknisa\GER\Controllers\TeknisaCrud {

    protected $dataSourceName = "search_pratodemanda";
    protected $dataSourceManager;
    protected $environment;

    public function __construct($crudService, $parameterBag, $dataSourceManager, $environment) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->dataSourceManager = $dataSourceManager;
        $this->environment = $environment;
    }

    public function searchPratoDemanda(Request\Filter $request, Response $response) {
        try {
            $onDemandFilter = $request->getFilterCriteria();
            $conditions = $onDemandFilter->getConditions();

            if (end($conditions)['columnName'] !== 'CDPRATONIVANTE') {
                if (end($conditions)['operator'] == 'LIKE_ALL' || end($conditions)['operator'] == 'LIKE_I') {
                    $filter = end($conditions)['value'];
                    
                    if(is_numeric(str_replace('%', '', $filter)))
                        $filter = str_replace('%', '', $filter).'%';
                        
                    $newConditions = array(
                        array(
                            "columnName" => "FILTER",
                            "operator" => "=",
                            "value" => $filter
                        )
                    );
                    $onDemandFilter = new FilterCriteria('search_pratofilter', $newConditions);
                }
                else {
                    if(end($conditions)['columnName'] != "CDPRATO" && end($conditions)['columnName'] != "NMPRATO"){    
                        $onDemandFilter->addCondition('CDNVPRATO', '1');
                    }
                    
                    $onDemandFilter->setDataSourceName($this->dataSourceName);
                }
            } else {
                $onDemandFilter->setDataSourceName($this->dataSourceName);
            }
            $onDemandFilter->addCondition("CDFILIAL", $this->environment->getCdFilial());
            $onDemandFilter->addCondition("NRORG", $this->environment->getNrOrg());
            $newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($onDemandFilter, array());
            $dataSource = $this->dataSourceManager->findBy($newFilterCriteria);
            $dataSet = self::montaDataSet($dataSource->getRows());
            $response->addDataSet(new DataSet('searchPratoDemanda', $dataSet));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function montaDataSet(array $dataSource) {
        $dataSet = array();

        foreach($dataSource as $data){
            if($data['PAI'] == 'true'){
                $data['PAI'] = true;
            } else{
                $data['PAI'] = false;
            }

            array_push($dataSet, array(
                'CDPRATO' => $data['CDPRATO'],
                'CDFILAUXPRAT' => $data['CDFILAUXPRAT'],
                'NMPRATO' => $data['NMPRATO'],
                'SGUNIDADE' => $data['SGUNIDADE'],
                'CDNVPRATO' => $data['CDNVPRATO'],
                'CDPRATONIVANTE' => $data['CDPRATONIVANTE'],
                '_hasChild' => $data['PAI']
            ));
        }
        return $dataSet;
    }
}